/*
 * TismTool  Tis Veugen, 2010-2011
 * Application, Simulation, Verification
 *
 * OSAL_MutexSem.c
 * version 2
 */
#include <string.h>
#include <assert.h>
#include "OSAL_Gen.h"
#include "OSAL_MutexSem.h"
#include <pthread.h>

#define ASSERT_MUTEX(h) assert((0<(h))&&((h)<=OSAL_NOF_MUTEXSEMS)&&(MutexSems[(h)-1].bOccupied))

typedef struct
{
  OSAL_Bool         bOccupied;
  pthread_mutex_t   cs;
} OSAL_MutexSem;

static pthread_mutex_t  MutexCs;
static OSAL_MutexSem    MutexSems[ OSAL_NOF_MUTEXSEMS ];

void OSAL_MutexSemInitialise( void )
{
  int rv;

  memset( MutexSems, 0, sizeof(MutexSems) );
  rv = pthread_mutex_init( &MutexCs, NULL );
  assert( rv == 0 );
}

void OSAL_MutexSemTerminate( void )
{
  pthread_mutex_destroy( &MutexCs );
}

OSAL_Handle OSAL_MutexSemCreate( void )
{
  OSAL_Handle h = OSAL_HANDLE_INVALID;
  int         i;
  int         rv;

  pthread_mutex_lock( &MutexCs );
  for ( i=0; i<OSAL_NOF_MUTEXSEMS; i++ )
  {
    if ( ! MutexSems[ i ].bOccupied )
    {
      rv = pthread_mutex_init( &MutexSems[ i ].cs, NULL );
      if ( rv == 0 )
      {
        MutexSems[ i ].bOccupied = TRUE;
        h = i+1;
      }
      break;
      /****/
    }
  }
  pthread_mutex_unlock( &MutexCs );

  return h;
}

void OSAL_MutexSemLock( OSAL_Handle h )
{
  ASSERT_MUTEX(h);
  pthread_mutex_lock( &MutexSems[ h-1 ].cs );
}

void OSAL_MutexSemUnlock( OSAL_Handle h )
{
  ASSERT_MUTEX(h);
  pthread_mutex_unlock( &MutexSems[ h-1 ].cs );
}

void OSAL_MutexSemReset( OSAL_Handle h )
{
  int         rv;

  ASSERT_MUTEX(h);
  pthread_mutex_destroy( &MutexSems[ h-1 ].cs );
  rv = pthread_mutex_init( &MutexSems[ h-1 ].cs, NULL );
  assert( rv == 0 );
}

void OSAL_MutexSemDelete( OSAL_Handle h )
{
  ASSERT_MUTEX(h);
  pthread_mutex_lock( &MutexCs );

  pthread_mutex_destroy( &MutexSems[ h-1 ].cs );
  MutexSems[ h-1 ].bOccupied = FALSE;

  pthread_mutex_unlock( &MutexCs );
}
