/*
 * TismTool  Tis Veugen, 2010-2011
 * Application, Simulation, Verification
 *
 * OSAL_RzVousSem.c
 * version 3
 */
#include <string.h>
#include <assert.h>
#include "OSAL_Gen.h"
#include "OSAL_RzVousSem.h"
#include <pthread.h>

#define ASSERT_RZVOUS(h) assert((0<(h))&&((h)<=OSAL_NOF_RZVOUSSEMS)&&(RzVousSems[(h)-1].bOccupied))

typedef struct
{
  OSAL_Bool       bOccupied;
  OSAL_Bool       bWait;
  pthread_mutex_t cs;
  pthread_cond_t  cv;
} OSAL_RzVousSem;

static pthread_mutex_t  RzVousCs;
static OSAL_RzVousSem   RzVousSems[ OSAL_NOF_RZVOUSSEMS ];

void OSAL_RzVousSemInitialise( void )
{
  int rv;

  memset( RzVousSems, 0, sizeof(RzVousSems) );
  rv = pthread_mutex_init( &RzVousCs, NULL );
  assert( rv == 0 );
}

void OSAL_RzVousSemTerminate( void )
{
  pthread_mutex_destroy( &RzVousCs );
}

OSAL_Handle OSAL_RzVousSemCreate( void )
{
  OSAL_Handle h = OSAL_HANDLE_INVALID;
  int         i;
  int         rv;

  pthread_mutex_lock( &RzVousCs );
  for ( i=0; i<OSAL_NOF_RZVOUSSEMS; i++ )
  {
    if ( ! RzVousSems[ i ].bOccupied )
    {
      rv = pthread_mutex_init( &RzVousSems[ i ].cs, NULL );
      if ( rv == 0 )
      {
        rv = pthread_cond_init( &RzVousSems[ i ].cv, NULL );
        if ( rv == 0 )
        {
          RzVousSems[ i ].bOccupied = TRUE;
          RzVousSems[ i ].bWait = FALSE;
          h = i+1;
        }
      }
      break;
      /****/
    }
  }
  pthread_mutex_unlock( &RzVousCs );

  return h;
}

void OSAL_RzVousSemWait( OSAL_Handle h )
{
  OSAL_RzVousSem* pRz;

  ASSERT_RZVOUS(h);
  pRz = &RzVousSems[ h-1 ];

  pthread_mutex_lock( &pRz->cs );
  if ( ! pRz->bWait )
  {
    pRz->bWait = TRUE;
    pthread_cond_wait( &pRz->cv, &pRz->cs );
  }
  pRz->bWait = FALSE;
  pthread_mutex_unlock( &pRz->cs );
}

void OSAL_RzVousSemRelease( OSAL_Handle h )
{
  OSAL_RzVousSem* pRz;

  ASSERT_RZVOUS(h);
  pRz = &RzVousSems[ h-1 ];

  pthread_mutex_lock( &pRz->cs );
  if ( pRz->bWait )
  {
    pthread_cond_signal( &pRz->cv );
  }
  else
  {
    pRz->bWait = TRUE;
  }
  pthread_mutex_unlock( &pRz->cs );
}

void OSAL_RzVousSemReset( OSAL_Handle h )
{
  OSAL_RzVousSem* pRz;

  ASSERT_RZVOUS(h);
  pRz = &RzVousSems[ h-1 ];

  pthread_mutex_lock( &pRz->cs );
  pRz->bWait = FALSE;
  pthread_mutex_unlock( &pRz->cs );
}

void OSAL_RzVousSemDelete( OSAL_Handle h )
{
  OSAL_RzVousSem* pRz;

  ASSERT_RZVOUS(h);
  pRz = &RzVousSems[ h-1 ];

  pthread_mutex_lock( &RzVousCs );

  pthread_cond_destroy( &pRz->cv );
  pthread_mutex_destroy( &pRz->cs );
  pRz->bOccupied = FALSE;
  pRz->bWait = FALSE;

  pthread_mutex_unlock( &RzVousCs );
}
