/*
 * TismTool © Tis Veugen, 2010-2011
 * Application, Simulation, Verification
 *
 * TsmArray.c
 * version 2
 */
#include <string.h>
#include "TsmGen.h"
#include "TsmArray.h"
#include "TsmException.h"
#include "TsmObject.h"

static void TsmArray_Inc( TsmArray* this )
{
  if ( this->Number >= this->Size )
  {
    if ( this->Incr == 0 )
    {
      TsmException_Quit( "TsmArray_Inc: full" );
    }
    else
    {
      int            NewSize = this->Size + this->Incr;
      TsmArrayValue* NewItems;

      if ( this->Size == 0 )
      {
        NewItems = (TsmArrayValue*) Tsm_Malloc( NewSize * sizeof(TsmArrayValue) );
        if ( NULL == NewItems )
        {
          TsmException_Quit( "TsmArray_Inc: Tsm_Malloc" );
        }
      }
      else
      {
        NewItems = (TsmArrayValue*) Tsm_Realloc( this->Items, NewSize * sizeof(TsmArrayValue) );
        if ( NULL == NewItems )
        {
          TsmException_Quit( "TsmArray_Inc: Tsm_Realloc" );
        }
      }
      this->Size = NewSize;
      this->Items = NewItems;
    }
  }
}

void TsmArray_Malloc( TsmArray* this, int Size, int Increment )
{
  this->Items = NULL;
  if ( Size > 0 )
  {
    this->Items = (TsmArrayValue*) Tsm_Malloc( Size * sizeof(TsmArrayValue) );
    if ( NULL == this->Items )
    {
      TsmException_Quit( "TsmArray_Malloc: Tsm_Malloc" );
    }
  }
  this->Size = Size;
  this->Incr = Increment;
  this->Number = 0;
}

void TsmArray_Free( TsmArray* this )
{
  if ( this->Size > 0 )
  {
    Tsm_Free( this->Items );
    this->Size = 0;
  }
}

int TsmArray_Add( TsmArray* this, TsmArrayValue Value )
{
  TsmArray_Inc( this );
  this->Items[ this->Number ] = Value;
  return this->Number++;
}

TsmArrayValue TsmArray_At( TsmArray* this, int i )
{
  if ( ( i < 0 ) || ( i >= this->Number ) )
  {
    TsmException_Quit( "TsmArray_At: index out of range" );
  }
  return this->Items[ i ];
}

void TsmArray_Clear( TsmArray* this )
{
  this->Number = 0;
}

void TsmArray_Insert( TsmArray* this, int i, TsmArrayValue Value )
{
  int             l;
  TsmArrayValue*  pSrc;
  TsmArrayValue*  pDst;

  if ( ( i < 0 ) || ( i > this->Number ) )
  {
    TsmException_Quit( "TsmArray_Insert: index out of range" );
  }

  TsmArray_Inc( this );

  /*
   * Don't use memcpy because of overlapping memory.
   */
  pSrc = &this->Items[ this->Number-1 ];
  pDst = &this->Items[ this->Number ];
  for ( l = this->Number - i; l > 0; l-- )
  {
    *pDst-- = *pSrc--;
  }

  this->Items[ i ] = Value;
  this->Number++;
}

void TsmArray_FreeItems( TsmArray* this )
{
  int i;

  for ( i=0; i<this->Number; i++ )
  {
    Tsm_Free( this->Items[i] );
  }
  this->Number = 0;
}

void TsmArray_DtorItems( TsmArray* this )
{
  int        i;
  TsmObject* p;

  for ( i=0; i<this->Number; i++ )
  {
    p = (TsmObject*) this->Items[i];
    p->Dtor( p );
  }
  this->Number = 0;
}
