/*
 * TismTool © Tis Veugen, 2010-2011
 * Application, Simulation, Verification
 *
 * TsmCompound.c
 * version 1
 */
#include <string.h>
#include <stdio.h>
#include "TsmGen.h"
#include "TsmCompound.h"
#include "TsmHandler.h"
#include "TsmRegion.h"
#include "TsmHandlerAction.h"
#include "TsmException.h"

TsmCompound* TsmCompound_Ctor( void )
{
  TsmCompound* this = (TsmCompound*) Tsm_Malloc( sizeof(TsmCompound) );
  TsmCompound_Ftor( this );
  return this;
}

void TsmCompound_Ftor( TsmCompound* this )
{
  TsmState_Ftor( (TsmState*) this );
  this->Class = "TsmCompound";
  this->mNrRegions = 0;
  this->mRegions = NULL;

  this->Gtor = (Tsm_pfObj) TsmCompound_Gtor;
  this->Initial = (Tsm_pfStHdlr) TsmCompound_Initial;
  this->ExecuteTrigger = (Tsm_pfStHdlr) TsmCompound_ExecuteTrigger;
  this->ExecuteTriggerCompound = TsmCompound_ExecuteTriggerCompound;
  this->AtMyExit = TsmCompound_AtMyExit;
  this->AtExit = (Tsm_pfStHdlr) TsmCompound_AtExit;
  this->TsmSnap = (void (*)( TsmState*, TsmStr )) TsmCompound_TsmSnap;
  this->TsmCheckVisit = (void (*)( TsmState* )) TsmCompound_TsmCheckVisit;
}

void TsmCompound_Gtor( TsmCompound* this )
{
  int i;

  for ( i=0; i<this->mNrRegions; i++ )
  {
    this->mRegions[i]->Dtor( (TsmObject*) this->mRegions[i] );
  }
}

static void TsmCompound_HandlerTakeOver( TsmCompound* this, TsmHandler* oHandler, TsmHandler* hAll )
{
  if ( (hAll->Action & TsmHandlerAction_TACTRANSITION) != 0 )
  {
    oHandler->Action    = TsmHandlerAction_TACTRANSITION;
    oHandler->Index     = hAll->Index;
    oHandler->TacSubFsm = hAll->TacSubFsm;
  }
  else
    if ( (hAll->Action & TsmHandlerAction_TRANSITION) != 0 )
    {
      oHandler->Action = TsmHandlerAction_TRANSITION;
      oHandler->Index  = hAll->Index;
    }
    else
      if ( (hAll->Action & TsmHandlerAction_OK) != 0 )
      {
        oHandler->Action = TsmHandlerAction_OK;
      }
      else
        if ( (hAll->Action & TsmHandlerAction_GUARD) != 0 )
        {
          oHandler->Action = TsmHandlerAction_GUARD;
        }
        else
          if ( hAll->Action == TsmHandlerAction_FINAL )
          {
            oHandler->Action = TsmHandlerAction_FINAL;
          }
          else
            if ( hAll->Action == TsmHandlerAction_NOK )
            {
              oHandler->Action = TsmHandlerAction_NOK;
            }
            else
              if ( TsmHandlerAction_IsFinalNok( hAll->Action ) )
              {
                oHandler->Action = TsmHandlerAction_NOK;
              }
              else
              {
                char* sException;

                oHandler->Action = TsmHandlerAction_NOK;

                sException = Tsm_Malloc( TSM_NCH_FULLNAME );
                sprintf( sException, "TismBUG, HandlerTakeOver action @ " );
                this->FullName( (TsmBase*) this, sException+strlen(sException) );
                TsmException_Quit( sException );
                Tsm_Free( sException );
              }
}

static void TsmCompound_ExecuteRegions( TsmCompound* this, TsmHandler* oHandler, TsmBool bTrigger )
{
  int         i;
  TsmHandler  hAll;
  TsmHandler  h;

  TsmHandler_Ftor( &hAll );
  for ( i=0; i<this->mNrRegions; i++ )
  {
    TsmRegion*  tRegion = this->mRegions[i];

    TsmHandler_Ftor1( &h, oHandler->Owner );

    if ( bTrigger )
    {
      h.Trigger = oHandler->Trigger;
      h.Prospect = oHandler->Prospect;
      tRegion->ExecuteTrigger( (TsmState*) tRegion, &h );
    }
    else
    {
      h.Action = TsmHandlerAction_ENTER;
      tRegion->ExecuteHandler( (TsmState*) tRegion, &h );
    }

    switch ( h.Action )
    {
    case TsmHandlerAction_TRANSITION:
      if ( TsmHandlerAction_HasTransition( hAll.Action ) )
      {
        char* sException;

        sException = Tsm_Malloc( TSM_NCH_FULLNAME );
        sprintf( sException, "9610: Multiple region transitions @ " );
        this->FullName( (TsmBase*) this, sException+strlen(sException) );
        TsmException_Quit( sException );
        Tsm_Free( sException );
      }
      hAll.Index = h.Index;
      break;
    case TsmHandlerAction_TACTRANSITION:
      if ( TsmHandlerAction_HasTransition( hAll.Action ) )
      {
        char* sException;

        sException = Tsm_Malloc( TSM_NCH_FULLNAME );
        sprintf( sException, "9611: Multiple region transitions @ " );
        this->FullName( (TsmBase*) this, sException+strlen(sException) );
        TsmException_Quit( sException );
        Tsm_Free( sException );
      }
      hAll.Index = h.Index;
      hAll.TacSubFsm = h.TacSubFsm;
      break;
    default:
      break;
    }
    hAll.Action |= h.Action;
    hAll.bTaken |= h.bTaken;
  }

  TsmCompound_HandlerTakeOver( this, oHandler, &hAll );
  oHandler->bTaken |= hAll.bTaken;
}

static void TsmCompound_ExecuteTriggerRegions( TsmCompound* this, TsmHandler* oHandler )
{
  TsmCompound_ExecuteRegions( this, oHandler, TRUE );
}

static void TsmCompound_ExitRegions( TsmCompound* this, TsmHandler* oHandler )
{
  int i;

  for ( i=0; i<this->mNrRegions; i++ )
  {
    this->mRegions[i]->AtExit( (TsmState*) this->mRegions[i], oHandler );
  }
}


void TsmCompound_Initial( TsmCompound* this, TsmHandler* oHandler )
{
  TsmCompound_ExecuteRegions( this, oHandler, FALSE );
}

void TsmCompound_ExecuteTrigger( TsmCompound* this, TsmHandler* oHandler )
{
  int ProspectSelf = this->GetProspect( (TsmState*) this, oHandler->Trigger->PortId, oHandler->Trigger->Command );

  oHandler->Prospect |= ProspectSelf;

  TsmCompound_ExecuteTriggerRegions( this, oHandler );

  if ( ! oHandler->bTaken )
  {
    /* Process the trigger itself, if transition available. */
    this->ExecuteTriggerCompound( this, oHandler );
  }
}

void TsmCompound_ExecuteTriggerCompound( TsmCompound* this, TsmHandler* oHandler )
{
  /* Override for all the compound triggers */
  oHandler->Action = TsmHandlerAction_NOK;
}

void TsmCompound_AtMyExit( TsmCompound* this, TsmHandler* oHandler )
{
}

void TsmCompound_AtExit( TsmCompound* this, TsmHandler* oHandler )
{
  TsmCompound_ExitRegions( this, oHandler );
  this->AtMyExit( this, oHandler );
}

void TsmCompound_TsmSnap( TsmCompound* this, TsmStr sout )
{
  int i;

  sprintf( sout, "[%s:", this->TsmName );
  for ( i=0; i<this->mNrRegions; i++ )
  {
    strcat( sout, "|" );
    this->mRegions[i]->TsmSnap( (TsmState*) this->mRegions[i], sout+strlen(sout) );
  }
  strcat( sout, "]" );
}

void TsmCompound_TsmCheckVisit( TsmCompound* this )
{
  int i;

  for ( i=0; i<this->mNrRegions; i++ )
  {
    this->mRegions[i]->TsmCheckVisit( (TsmState*) this->mRegions[i] );
  }
}
