/*
 * TismTool © Tis Veugen, 2010-2011
 * Simulation, Verification
 *
 * TsmState.c
 * version 2
 */
#include <stdio.h>
#include <string.h>
#include "Tsm.h"
#include "TsmGen.h"
#include "TsmBase.h"
#include "TsmState.h"
#include "TsmTransition.h"
#include "TsmHandler.h"
#include "TsmHandlerAction.h"
#include "TsmException.h"
#include "TsmProspect.h"
#include "TsmRecorder.h"

TsmState* TsmState_Ctor( void )
{
  TsmState* this = (TsmState*) Tsm_Malloc( sizeof(TsmState) );
  TsmState_Ftor( this );
  return this;
}

void TsmState_Ftor( TsmState* this )
{
  TsmBase_Ftor( (TsmBase*) this );
  this->Class = "TsmState";
  this->bVisit = FALSE;
  this->mNrTransitions = 0;
  this->mTransitions = NULL;

  this->Gtor = (Tsm_pfObj) TsmState_Gtor;
  this->AtEntry = TsmState_AtEntry;
  this->AtExit = TsmState_AtExit;
  this->Initial = TsmState_Initial;
  this->Entrypoint = TsmState_Entrypoint;
  this->ExecuteTrigger = TsmState_ExecuteTrigger;
  this->ExecuteTriggerless = TsmState_ExecuteTriggerless;
  this->ExecuteTransition = TsmState_ExecuteTransition;
  this->ExecuteHandler = TsmState_ExecuteHandler;
  this->GetProspect = TsmState_GetProspect;
  this->TsmSnap = TsmState_TsmSnap;
  this->TsmCheckVisit = TsmState_TsmCheckVisit;
}

void TsmState_Gtor( TsmState* this )
{
  int i;
  if ( this->mNrTransitions > 0 )
  {
    for ( i=0; i<this->mNrTransitions; i++ )
    {
      TsmTransition_Gtor( this->mTransitions[i] );
    }
  }
}

void TsmState_AtEntry( TsmState* this, TsmHandler* oHandler ) {}
void TsmState_AtExit( TsmState* this, TsmHandler* oHandler ) {}
void TsmState_Initial( TsmState* this, TsmHandler* oHandler ) {}
void TsmState_Entrypoint( TsmState* this, TsmHandler* oHandler ) {}

void TsmState_ExecuteTrigger( TsmState* this, TsmHandler* oHandler )
{
  /* N.B. Some states may have no triggers */
  oHandler->Action = TsmHandlerAction_NOK;
}

void TsmState_ExecuteTriggerless( TsmState* this, TsmHandler* oHandler )
{
  oHandler->Action = TsmHandlerAction_NOK;
}

TsmBool TsmState_ExecuteTransition( TsmState* this, TsmHandler* oHandler )
{
  return TRUE;
}

void TsmState_ExecuteHandler( TsmState* this, TsmHandler* oHandler )
{
  TsmBool bReady = FALSE;

  do
  {
    switch ( oHandler->Action )
    {
    case TsmHandlerAction_TRANSITION:
    case TsmHandlerAction_TACTRANSITION:
      bReady = this->ExecuteTransition( this, oHandler );
      break;
    case TsmHandlerAction_EXITPOINT:
      oHandler->Action = TsmHandlerAction_TRANSITION;
      bReady = TRUE;
      break;
    case TsmHandlerAction_ENTER:
      this->AtEntry( this, oHandler );
      this->Initial( this, oHandler );
      break;
    case TsmHandlerAction_INITIAL:
      this->Initial( this, oHandler );
      break;
    case TsmHandlerAction_ENTRYPOINT:
      this->AtEntry( this, oHandler );
      this->Entrypoint( this, oHandler );
      break;
    case TsmHandlerAction_OK:
    case TsmHandlerAction_GUARD:
    case TsmHandlerAction_NOK:
      bReady = TRUE;
      break;
    case TsmHandlerAction_FINAL:
    case TsmHandlerAction_QUIT:
      this->ExecuteTriggerless( this, oHandler );
      bReady = TRUE;
      break;
    default :
      {
        char* sException;

        sException = Tsm_Malloc( TSM_NCH_FULLNAME );
        sprintf( sException, "TsmHandlerAction '%d' @ ", oHandler->Action );
        this->FullName( (TsmBase*) this, sException+strlen(sException) );
        TsmException_Quit( sException );
        Tsm_Free( sException );
      }
      break;
    }
  } while ( ! bReady );
}

int TsmState_GetProspect( TsmState* this, int PortId, int Command )
{
  return TsmProspect_NOK;
}

void TsmState_TsmSnap( TsmState* this, TsmStr sout )
{
  strcpy( sout, this->TsmName );
}

void TsmState_TsmCheckVisit( TsmState* this )
{
  char* sName = Tsm_Malloc( TSM_NCH_FULLNAME );
  int   i;

  if ( ! this->bVisit )
  {
    this->FullName( (TsmBase*) this, sName );
    TsmRecorder_UnvisitedState( sName );
  }

  for ( i=0; i<this->mNrTransitions; i++ )
  {
    if ( ! this->mTransitions[i]->bVisit )
    {
      this->FullName( (TsmBase*) this, sName );
      TsmRecorder_UnvisitedTransition( sName, this->mTransitions[i]->FromTo );
    }
  }
  Tsm_Free( sName );
}

void TsmState_Lgl( TsmState* this, int iLevel, char* s )
{
  char* sName = Tsm_Malloc( TSM_NCH_FULLNAME );

  strcpy( sName, s );
  this->FullName( (TsmBase*) this, sName+strlen(sName) );
  Tsm_Lgl( iLevel, sName );

  Tsm_Free( sName );
}
