/*
 * TismTool © Tis Veugen, 2010-2011
 * Simulation, Verification
 *
 * TsmSupervisor.c
 * version 2
 */
#include "OSAL_Gen.h"
#include "OSAL_Thread.h"
#include "Tsm.h"
#include "TsmGen.h"
#include "TsmSupervisor.h"
#include "TsmMutex.h"
#include "TsmActivity.h"
#include "TsmCbThread.h"
#include "TsmThread.h"
#include "TsmThreadState.h"
#include "TsmThreadType.h"
#include "TsmSvrHelpers.h"
#include "TsmSvrChecker.h"

static TsmMutex     mLockThreads;
static TsmActivity  mActivity;
static TsmCbThread  mCbThread;
static TsmBool      mbVerify = FALSE;

static int          mMaximumQueueSize = 6;

int       TsmSupervisor_mRandomSeed = 1;
long      TsmSupervisor_mMaxNrSimLoops = 1000;

void TsmSupervisor_Initialise( void )
{
  Tsm_Open();
  TsmMutex_Ftor( &mLockThreads );
  TsmActivity_Ftor( &mActivity );
  TsmSupervisor_Register2( NULL, TsmThreadType_SUPERVISOR );

  TsmCbThread_Ftor( &mCbThread );
  mCbThread.Prepare( (TsmThread*) &mCbThread, "TismRTE" );
  mCbThread.Run( (TsmThread*) &mCbThread );
}

void TsmSupervisor_Terminate( void )
{
  mCbThread.Halt( (TsmThread*) &mCbThread );
  mCbThread.Cease( (TsmThread*) &mCbThread );
  Tsm_Close();
}

TsmIntQueue* TsmSupervisor_GetIntQueue( void )
{
  return mCbThread.GetIntQueue( (TsmThread*) &mCbThread );
}

void TsmSupervisor_Announce( void )
{
  mActivity.Announce( &mActivity );
}

void TsmSupervisor_Abandon( void )
{
  mActivity.Abandon( &mActivity );
}

TsmArray* TsmSupervisor_GetClasses( void )
{
  TsmSvrChecker* Checker = TsmSvrChecker_Instance();

  return TsmSvrChecker_GetClasses( Checker );
}

int TsmSupervisor_Register( TsmThread* oTsmThread )
{
  return TsmSupervisor_Register2( oTsmThread, TsmThreadType_THREAD );
}

int TsmSupervisor_Register2( TsmThread* oTsmThread, TsmThreadType eType )
{
  OSAL_Handle    lThreadBid;
  TsmSvrChecker* Checker = TsmSvrChecker_Instance();
  
  if ( NULL != oTsmThread )
  {
    lThreadBid = oTsmThread->GetThread( oTsmThread );
  }
  else
  {
    lThreadBid = OSAL_GetCurrentThreadHandle();
  }

  return TsmSvrChecker_Register( Checker, oTsmThread, eType, lThreadBid );
}

void TsmSupervisor_SetState( OSAL_Handle ThreadBid )
{
  TsmSupervisor_SetState2( ThreadBid, TsmThreadState_RUNNING );
}

void TsmSupervisor_SetState2( OSAL_Handle ThreadBid, TsmThreadState eState )
{
  TsmSvrChecker* Checker = TsmSvrChecker_Instance();

  TsmSvrChecker_SetState( Checker, ThreadBid, eState );
}

void TsmSupervisor_SetStateRv( OSAL_Handle ThreadBid, TsmThread* oRvThread )
{
  TsmSvrChecker* Checker = TsmSvrChecker_Instance();

  TsmSvrChecker_SetStateRv( Checker, ThreadBid, oRvThread );
}

void TsmSupervisor_NativeEntry( TsmClass* oOwner, const int aNatives[], int NrNatives )
{
  TsmSvrChecker* Checker = TsmSvrChecker_Instance();

  TsmSvrChecker_NativeEntry( Checker, oOwner, aNatives, NrNatives );
}

void TsmSupervisor_NativeExit( TsmClass* oOwner, const int aNatives[], int NrNatives )
{
  TsmSvrChecker* Checker = TsmSvrChecker_Instance();

  TsmSvrChecker_NativeExit( Checker, oOwner, aNatives, NrNatives );
}

void TsmSupervisor_SnappedClasses( TsmStr sout )
{
  TsmSvrChecker* Checker = TsmSvrChecker_Instance();

  TsmSvrChecker_SnappedClasses( Checker, sout );
}

void TsmSupervisor_SnappedThreads( TsmStr sout )
{
  TsmSvrChecker* Checker = TsmSvrChecker_Instance();

  TsmSvrChecker_SnappedThreads( Checker, sout );
}

void TsmSupervisor_GetCurrentSituation( TsmStr sout )
{
  TsmSvrChecker* Checker = TsmSvrChecker_Instance();

  TsmSvrChecker_GetCurrentSituation( Checker, sout );
}

/*
 * Called by Queues.
 */
TsmBool TsmSupervisor_QueueOverflow( int QueueCount )
{
  TsmBool bOverflow = FALSE;

  TsmMutex_WaitOne( &mLockThreads );
  if ( QueueCount >= mMaximumQueueSize )
  {
    TsmSvrChecker* Checker = TsmSvrChecker_Instance();

    TsmSvrChecker_AddQueueOverflow( Checker );
    bOverflow = TRUE;
  }
  TsmMutex_Release( &mLockThreads );

  return bOverflow;
}

/*
 * Called by Fsms.
 */
TsmBool TsmSupervisor_ChoiceOK( int MaxNrChoices, int ChoiceIndex, char* sGuard )
{
  TsmBool        bChoice;
  TsmSvrChecker* Checker = TsmSvrChecker_Instance();

  TsmMutex_WaitOne( &mLockThreads );
  if ( mbVerify )
  {
    bChoice = TsmSvrChecker_ChoiceOKVerify( Checker, MaxNrChoices, ChoiceIndex, sGuard );
  }
  else
  {
    bChoice = TsmSvrChecker_ChoiceOKSimulate( Checker, MaxNrChoices, ChoiceIndex );
  }
  TsmMutex_Release( &mLockThreads );

  return bChoice;
}


void TsmSupervisor_Verify( void )
{
  TsmSvrChecker* Checker = TsmSvrChecker_Instance();

  mbVerify = TRUE;
  TsmSvrChecker_Verify( Checker );
}

void TsmSupervisor_ConfigureSimulation( int iSeed, long iNrLoops )
{
  TsmSupervisor_mRandomSeed = iSeed;
  TsmSupervisor_mMaxNrSimLoops = iNrLoops;
}

void TsmSupervisor_Simulate( void )
{
  TsmSvrChecker* Checker = TsmSvrChecker_Instance();

  mbVerify = FALSE;
  TsmSvrChecker_Simulate( Checker );
}

void TsmSupervisor_Summary( void )
{
  TsmSvrChecker* Checker = TsmSvrChecker_Instance();

  TsmSvrChecker_Summary( Checker, mbVerify );
}

void TsmSupervisor_Cleanup( void )
{
  TsmSvrChecker_Cleanup();
}
