/*
 * TismTool © Tis Veugen, 2010-2011
 * Simulation, Verification
 *
 * TsmSvrHelpers.h
 * version 1
 */
#ifndef TSM_SVRHELPERS_H
#define TSM_SVRHELPERS_H

#ifdef __cplusplus
extern "C" {
#endif

#include "TsmThreadType.h"
#include "TsmThread.h"
#include "TsmThreadState.h"
#include "Tsm.h"
#include "TsmGen.h"
#include "TsmClass.h"
#include "TsmArray.h"

/******************** TsmThreadData ********************/

typedef struct TsmThreadDataStruct
{
  TsmThreadType  Type;
  int            ThreadId;
  TsmThread*     TsmThread;
  TsmThreadState State;
  TsmThread*     RvThread;
  TsmBool        bInterrupt;
} TsmThreadData;

TsmBool TsmThreadData_HasMessage( TsmThreadData* this );
void    TsmThreadData_ToState( TsmThreadData* this, TsmStr sout );

/******************** TsmSnapshot ********************/

typedef struct TsmSnapshotStruct
{
  struct TsmSnapshotStruct* Parent;
  char*                     SnapClasses;
  char*                     SnapCbThread;
  char*                     SnapThreads;
} TsmSnapshot;

TsmSnapshot*  TsmSnapshot_Ctor( char* sSnapClasses, char* sSnapCbThread, char* sSnapThreads );
void          TsmSnapshot_Ftor( TsmSnapshot* this, char* sSnapClasses, char* sSnapCbThread, char* sSnapThreads );
void          TsmSnapshot_Gtor( TsmSnapshot* this );
void          TsmSnapshot_Dtor( TsmSnapshot* this );
void          TsmSnapshot_ToString( TsmSnapshot* this, TsmStr sout );
TsmBool       TsmSnapshot_Equals( TsmSnapshot* this, void* o2 );

/******************** TsmScenItemKind ********************/

typedef enum TsmScenItemKindEnum
{
  TsmScenItemKind_RELEASE_CB_MSG,
  TsmScenItemKind_RELEASE_CB_RV2,
  TsmScenItemKind_RELEASE_MSG,
  TsmScenItemKind_RELEASE_RV2,
  TsmScenItemKind_NATIVE,
  TsmScenItemKind_CHOICE,
  TsmScenItemKind_QO
} TsmScenItemKind;

/******************** TsmScenItem ********************/

typedef struct TsmScenItemStruct
{
#include "TsmObject_fields.h"
  int              RefCount;
  TsmScenItemKind  Kind;
  int              Index;
  TsmClass*        Receiver;
  TsmThreadData*   ThreadData;
} TsmScenItem;

TsmScenItem*  TsmScenItem_Ctor( void );
void          TsmScenItem_Ftor( TsmScenItem* this );
void          TsmScenItem_Dtor( TsmScenItem* this );
void          TsmScenItem_AddRef( TsmScenItem* this );
void          TsmScenItem_ToString( TsmScenItem* this, TsmStr sout );

/******************** TsmScenario ********************/

typedef struct TsmScenarioStruct
{
  struct TsmScenarioStruct* pNext;  /* for TsmStackList */
  int                       Id;
  struct TsmScenarioStruct* Parent;
  TsmArray                  Items;
  TsmSnapshot*              Snapshot;
} TsmScenario;

void         TsmScenario_NewScenario( TsmScenario* this, TsmScenario* oParent );
TsmScenario* TsmScenario_Ctor( void );
TsmScenario* TsmScenario_Ctor2( TsmScenario* oParent, TsmArray* oItems );
void         TsmScenario_Ftor2( TsmScenario* this, TsmScenario* oParent, TsmArray* oItems );
void         TsmScenario_Gtor( TsmScenario* this );
void         TsmScenario_Dtor( TsmScenario* this );
int          TsmScenario_GetParentId( TsmScenario* this );
TsmScenario* TsmScenario_Extend( TsmScenario* this, TsmScenItem* oTsmScenItem );
TsmScenario* TsmScenario_ExtendChoice( TsmScenario* this, TsmScenItem* oTsmScenItem );
void         TsmScenario_Add( TsmScenario* this, TsmScenItem* oTsmScenItem );
void         TsmScenario_Update( TsmScenario* this, TsmSnapshot* oSnapshot );
TsmBool      TsmScenario_HasCycle( TsmScenario* this );
void         TsmScenario_ToString( TsmScenario* this, TsmStr sout );

/******************** TsmSnapdata ********************/

typedef struct TsmSnapdataStruct
{
  int          NrRecords;
  int          NrCalls;
  TsmScenario* Scenario;
} TsmSnapdata;

TsmSnapdata*  TsmSnapdata_Ctor( int iNrRecords, int iNrCalls, TsmScenario* oScenario );
void          TsmSnapdata_Ftor( TsmSnapdata* this, int iNrRecords, int iNrCalls, TsmScenario* oScenario );

#ifdef __cplusplus
}
#endif 
#endif
